# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .budget_tags import BudgetTags

__all__ = ["TagDeleteResponse"]

TagDeleteResponse = List[BudgetTags]
